###################################################
### chunk number 1: 
###################################################
library(gmm)


###################################################
### chunk number 2: 
###################################################
g1 <- function(tet,x)
        {
        m1 <- (tet[1]-x)
        m2 <- (tet[2]^2 - (x - tet[1])^2)
        m3 <- x^3-tet[1]*(tet[1]^2+3*tet[2]^2)
        f <- cbind(m1,m2,m3)
        return(f)
        }


###################################################
### chunk number 3: 
###################################################
Dg <- function(tet,x)
        {
        G <- matrix(c( 1,
                              2*(-tet[1]+mean(x)),
                              -3*tet[1]^2-3*tet[2]^2,0,
                              2*tet[2],-6*tet[1]*tet[2]),
                           nrow=3,ncol=2)
        return(G)
        }


###################################################
### chunk number 4: 
###################################################
set.seed(123)
n <- 200
x1 <- rnorm(n, mean = 4, sd = 2)
print(mu <- mean(x1))
print(sig <- sqrt(var(x1)*(n-1)/n))


###################################################
### chunk number 5: 
###################################################
print(res <- gmm(g1,x1,c(0,0),grad=Dg))


###################################################
### chunk number 6: 
###################################################
summary(res)


###################################################
### chunk number 7: 
###################################################
sim_ex <- function(n,iter)
    {
    tet1 <- matrix(0,iter,2)
    tet2 <- tet1 
    for(i in 1:iter)
      {
      x1 <- rnorm(n, mean = 4, sd = 2)
      tet1[i,1] <- mean(x1)
      tet1[i,2] <- sqrt(var(x1)*(n-1)/n)
      tet2[i,] <- gmm(g1,x1,c(0,0),grad=Dg)$coefficients
      }
    bias <- cbind(rowMeans(t(tet1)-c(4,2)),rowMeans(t(tet2)-c(4,2)))
    dimnames(bias)<-list(c("mu","sigma"),c("ML","GMM"))
    Var <- cbind(diag(var(tet1)),diag(var(tet2)))
    dimnames(Var)<-list(c("mu","sigma"),c("ML","GMM"))
    MSE <- cbind(rowMeans((t(tet1)-c(4,2))^2),rowMeans((t(tet2)-c(4,2))^2))
    dimnames(MSE)<-list(c("mu","sigma"),c("ML","GMM"))
    return(list(bias=bias,Variance=Var,MSE=MSE))
    }


###################################################
### chunk number 8: 
###################################################
g2 <- function(theta,x)
	{
	tau <- seq(1,5,length.out=10)
	pm <- 1
	x <- matrix(c(x),ncol=1)
	x_comp <- x%*%matrix(tau,nrow=1)
	x_comp <- matrix(complex(ima=x_comp),ncol=length(tau))
	emp_car <- exp(x_comp)
	the_car <- charStable(theta,tau,pm)
	gt <- t(t(emp_car) - the_car)
	gt <- cbind(Im(gt),Re(gt))
	return(gt)
	}


###################################################
### chunk number 9: 
###################################################
library(fBasics)
set.seed(345)
x2 <- rstable(500,1.5,.5,pm=1)
print(res <- gmm(g2,x2,t0<-c(2,0,sd(x2)/sqrt(2),0)))


###################################################
### chunk number 10: 
###################################################
summary(res)


###################################################
### chunk number 11: 
###################################################
res2 <- gmm(g2,x2,t0,optfct="nlminb",lower=c(0,-1,0,-Inf),upper=c(2,1,Inf,Inf))
summary(res2)


###################################################
### chunk number 12: 
###################################################
data(Finance)
x3 <- Finance[1:500,"WMK"]
res3 <- gmm(g2,x3,c(2,0,sd(x3)/sqrt(2),0),optfct="nlminb",lower=c(0,-1,0,-Inf),upper=c(2,1,Inf,Inf))
summary(res3)


###################################################
### chunk number 13: 
###################################################
library(car)
linear.hypothesis(res3,cbind(diag(2),c(0,0),c(0,0)),c(2,0))


###################################################
### chunk number 14: 
###################################################
library(mvtnorm)
sig <- matrix(c(1,.5,.5,1),2,2)
n <- 400
e <- rmvnorm(n,sigma=sig)
x4 <- rnorm(n)
w <- exp(-x4^2) + e[,1]
y <- 0.1*w + e[,2]


###################################################
### chunk number 15: 
###################################################
h <- cbind(x4, x4^2, x4^3)
g3 <- y~w


###################################################
### chunk number 16: 
###################################################
summary(res <- gmm(g3,x=h))


###################################################
### chunk number 17: 
###################################################
res2 <- gmm(g3,x=h,type='iterative')
summary(res2)$coef


###################################################
### chunk number 18: 
###################################################
res3 <- gmm(g3,x=h,res2$pcoef,type='cue')
summary(res3)$coef


###################################################
### chunk number 19: 
###################################################
confint(res3,level=.90)


###################################################
### chunk number 20: 
###################################################
plot(w,y,main="LS vs GMM estimation")
lines(w,fitted(res),col=2)
lines(w,fitted(lm(y~w)),col=3)
legend("topleft",c("Data","Fitted GMM","Fitted LS"),col=1:3,lty=c(1,1,1))


###################################################
### chunk number 21: 
###################################################
t <- 400
set.seed(345)
x5 <- arima.sim(n=t,list(ar=c(1.4,-0.6),ma=c(0.6,-0.3)))
xt <- x5[7:t]
xt1 <- x5[6:(t-1)]
xt2 <- x5[5:(t-2)]
h2 <- cbind(x5[4:(t-3)],x5[3:(t-4)],x5[2:(t-5)],x5[1:(t-6)])
g4 <- xt~xt1+xt2
res <- gmm(g4,x=h2)
summary(res)


###################################################
### chunk number 22: 
###################################################
res2 <- gmm(g4,x=h2,kernel="Truncated")
coef(res2)
res3 <- gmm(g4,x=h2,kernel="Bartlett")
coef(res3)
res4 <- gmm(g4,x=h2,kernel="Parzen")
coef(res4)
res5<- gmm(g4,x=h2,kernel="Tukey-Hanning")
coef(res5)


###################################################
### chunk number 23: 
###################################################
diag(vcov(res2))^.5
diag(vcov(res3))^.5
diag(vcov(res4))^.5
diag(vcov(res5))^.5


###################################################
### chunk number 24: 
###################################################
plot(res,which=2)


###################################################
### chunk number 25: 
###################################################
plot(res,which=3)


###################################################
### chunk number 26: 
###################################################
data(Finance)
r <- Finance[1:500,1:5]
rm <- Finance[1:500,"rm"]
rf <- Finance[1:500,"rf"]
z <- as.matrix(r-rf)
zm <- as.matrix(rm-rf)
res <- gmm(z~zm,x=zm)
summary(res)$coef
R <- cbind(diag(5),matrix(0,5,5))
c <- rep(0,5)
linear.hypothesis(res,R,c,test = "Chisq")


###################################################
### chunk number 27:  eval=FALSE
###################################################
## test <- paste(names(coef(res)[1:5])," = 0",sep="")
## linear.hypothesis(res,test)


###################################################
### chunk number 28: 
###################################################
res2<-gmm(z~zm-1,cbind(1,zm))
summary(res2)$j


###################################################
### chunk number 29: 
###################################################
g5 <- function(tet, x) {
     gmat <- (tet[1] + tet[2] * (1 + c(x[, 1]))) * (1 + x[, 2:6]) -  1
     return(gmat)
 }
res_sdf <- gmm(g5, x = as.matrix(cbind(rm, r)), c(0, 0))
summary(res_sdf)$j


###################################################
### chunk number 30: 
###################################################
g6 <- function(theta, x) {
     t <- length(x)
     et1 <- diff(x) - theta[1] - theta[2] * x[-t]
     ht <- et1^2 - theta[3] * x[-t]^(2 * theta[4])
     g <- cbind(et1, et1 * x[-t], ht, ht * x[-t])
     return(g)
 }


###################################################
### chunk number 31: 
###################################################
rf <- Finance[,"rf"]
rf <- ((1 + rf/100)^(365) - 1) * 100
dr <- diff(rf)
res_0 <- lm(dr ~ rf[-length(rf)])
tet0 <- c(res_0$coef, var(residuals(res_0)), 0)
res_rf <- gmm(g6, rf, tet0, control = list(maxit = 1000, reltol = 1e-10))
summary(res_rf)$coef


###################################################
### chunk number 32:  eval=FALSE
###################################################
## y <- rbind(y1-mean(y1),y2-mean(y2),y3-mean(y3))
## x <- rbind(x1-mean(x1),x2-mean(x2),x3-mean(x3))
## res <- gmm(y~x,h)


###################################################
### chunk number 33:  eval=FALSE
###################################################
## y <- rbind(y1,y2,y3)
## x <- rbind(x1,x2,x3)
## res <- gmm(y~x,h)


###################################################
### chunk number 34: 
###################################################
res_el <- gel(g1,x1,c(mean(x1),sd(x1)))
summary(res_el)


###################################################
### chunk number 35: 
###################################################
res_et <- gel(g1,x1,c(mean(x1),sd(x1)),type="ET")
summary(res_et)$coef


###################################################
### chunk number 36: 
###################################################
res_cue <- gel(g1,x1,c(mean(x1),sd(x1)),type="CUE")
summary(res_cue)$coef


###################################################
### chunk number 37: 
###################################################
res <- gel(g1,x1,c(mean(x1),sd(x1)),type="ETEL")
summary(res)$coef


###################################################
### chunk number 38: 
###################################################
tet0 <- gmm(g4,x=h2,wmatrix="ident")$coef
res <- gel(g4,x=h2,tet0,smooth=TRUE,kernel="Truncated")
summary(res)


###################################################
### chunk number 39: 
###################################################
plot(res,which=4)


###################################################
### chunk number 40:  eval=FALSE
###################################################
## ui=cbind(0,-1,-1)
## ci <- -1


###################################################
### chunk number 41:  eval=FALSE
###################################################
## res <- gel(g4,x=h2,tet0,smooth=TRUE,kernel="Truncated",constraint=TRUE, ui=ui,ci=ci)


###################################################
### chunk number 42:  eval=FALSE
###################################################
## res <- gel(g4,x=h2,tet0,smooth=TRUE,kernel="Truncated",optlam="numeric")


